import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.Vector;

import org.xml.sax.*;
import com.ibm.xml.parsers.*;

public class DHLTrackParser extends HandlerBase
{
  private DHLSummaryRecord summaryRecord;
  private DHLServiceEvent serviceEvent;
  boolean isSummaryRecord,isEventRecord,isOrigin, isDestination;
  String elementName = "", elementValue = "", awbNumber = "";
  Vector serviceEvents = new Vector();

  public DHLSummaryRecord getSummaryRecord() {
     return summaryRecord;
  }

  public Vector getServiceEvents() {
     return serviceEvents;
  }

  public void parseURI(String uri)
  {
    SAXParser parser = new SAXParser();
    parser.setDocumentHandler(this);
    parser.setErrorHandler(this);

    StringReader sr =
      new StringReader(uri);
    InputSource iSrc = new InputSource(sr);

    try
    {
      parser.parse(iSrc);
    }
    catch (Exception e)
    {
      System.err.println(e);
    }
  }

  public void startElement(String name, AttributeList attrs)
  {
    if(name.equals ("SUMMARY_RECORD")) {
       summaryRecord = new DHLSummaryRecord();
       if(attrs != null) summaryRecord.setAwbNumber (attrs.getValue (0));
       isSummaryRecord = true;
    }
    else if(name.equals ("EVENT_RECORD")) {
       if(attrs != null) awbNumber = attrs.getValue (0);
       isEventRecord = true;
    }
    else if(name.equals ("SERVICE_EVENTS")){
        serviceEvent = new DHLServiceEvent();
        serviceEvents.addElement (serviceEvent);
        serviceEvent.setAwbNumber (awbNumber);
    }
    else if(name.equals ("ORIGIN"))
       isOrigin = true;
    else if(name.equals ("DESTINATION"))
       isDestination = true;
    elementName = name;
    elementValue="";
  } // startElement(String,AttributeList)

  public void characters(char ch[], int start, int length)
  {
     elementValue = new String(ch,start,length);
  } // characters(char[],int,int);

  public void endElement(String name)
  {
    if(elementName.equals (name) && isSummaryRecord) {
       if(name.equals ("CITY")) {
          if(isOrigin) summaryRecord.setOriginCity (elementValue);
          else if(isDestination) summaryRecord.setDestiCity (elementValue);
       }
       else if(name.equals ("COUNTRY")) {
          if(isOrigin) summaryRecord.setOriginCountry (elementValue);
          else if(isDestination) summaryRecord.setDestiCountry (elementValue);
       }
       else if(name.equals ("MONTH"))
          summaryRecord.setLastEventMonth(elementValue);
       else if(name.equals ("DAY"))
          summaryRecord.setLastEventDay (elementValue);
       else if(name.equals ("YEAR"))
          summaryRecord.setLastEventYear (elementValue);
       else if(name.equals ("HOURS"))
          summaryRecord.setLastEventHour (elementValue);
       else if(name.equals ("MINUTES"))
          summaryRecord.setLastEventMinute (elementValue);
       else if(name.equals ("SIGNATORY"))
          summaryRecord.setSignatory (elementValue);
       else if(name.equals ("LAST_EVENT_COMMENTS"))
          summaryRecord.setLastEventComment (elementValue);
       else if(name.equals ("NODATAINFO"))
          summaryRecord.setNoDataInfo (elementValue);
    }
    else if(elementName.equals (name) && isEventRecord) {
       if(name.equals ("MONTH"))
          serviceEvent.setEventMonth(elementValue);
       else if(name.equals("DAY"))
          serviceEvent.setEventDay(elementValue);
       else if(name.equals("YEAR"))
          serviceEvent.setEventYear(elementValue);
       else if(name.equals("HOURS"))
          serviceEvent.setEventHour(elementValue);
       else if(name.equals("MINUTES"))
          serviceEvent.setEventMinute(elementValue);
       else if(name.equals("CITY"))
          serviceEvent.setEventCity(elementValue);
       else if(name.equals("COUNTRY"))
          serviceEvent.setEventCountry(elementValue);
       else if(name.equals("EVENT_COMMENTS"))
          serviceEvent.setEventComments(elementValue);
    }

    if(name.equals ("SUMMARY_RECORD"))
       isSummaryRecord = false;
    else if(name.equals ("EVENT_RECORD")) {
       isEventRecord = false;
       awbNumber = null;
    }
    else if(name.equals ("ORIGIN"))
       isOrigin = false;
    else if(name.equals ("DESTINATION"))
       isDestination = false;

    elementName = "";
    elementValue = "";
  } // endElement(String)

  public void endDocument()
  {
  } // endDocument()

  public void warning(SAXParseException ex)
  {
    System.err.println("[Warning] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  public void error(SAXParseException ex)
  {
    System.err.println("[Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  public void fatalError(SAXParseException ex)
    throws SAXException
  {
    System.err.println("[Fatal Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
    throw ex;
  }

  private String getLocationString(SAXParseException ex)
  {
    StringBuffer str = new StringBuffer();

    String systemId = ex.getSystemId();
    if (systemId != null)
    {
      int index = systemId.lastIndexOf('/');
      if (index != -1)
        systemId = systemId.substring(index + 1);
      str.append(systemId);
    }
    str.append(':');
    str.append(ex.getLineNumber());
    str.append(':');
    str.append(ex.getColumnNumber());

    return str.toString();
  } // getLocationString(SAXParseException):String
}

